Index: xdata/xodtemplate.c
===================================================================
--- xdata/xodtemplate.c	(revision 2574)
+++ xdata/xodtemplate.c	(working copy)
@@ -1309,6 +1309,10 @@
 				if((temp_command->command_line = (char *)strdup(value)) == NULL)
 					result = ERROR;
 				}
+			else if(!strcmp(variable, "env")) {
+				if(xodtemplate_add_commandenvironment(temp_command, value) == NULL)
+					result = ERROR;
+				}
 			else if(!strcmp(variable, "register"))
 				temp_command->register_object = (atoi(value) > 0) ? TRUE : FALSE;
 			else {
@@ -3547,7 +3551,30 @@
 	}
 
 
+/* adds an envvar to a command definition */
+xodtemplate_commandenvironment *xodtemplate_add_commandenvironment(xodtemplate_command *command, char *env) {
+	xodtemplate_commandenvironment *new_commandenvironment = NULL;
 
+	/* make sure we have the data we need */
+	if(env == NULL || !strcmp(env, ""))
+		return NULL;
+
+	/* allocate memory for a new env */
+	if((new_commandenvironment = malloc(sizeof(xodtemplate_commandenvironment))) == NULL)
+		return NULL;
+	if((new_commandenvironment->env = (char *)strdup(env)) == NULL) {
+		my_free(new_commandenvironment);
+		return NULL;
+		}
+
+	/* add the new member to the head of the member list */
+	new_commandenvironment->next = command->env;
+	command->env = new_commandenvironment;
+
+	return new_commandenvironment;
+	}
+
+
 /* parses a timeperod directive... :-) */
 int xodtemplate_parse_timeperiod_directive(xodtemplate_timeperiod *tperiod, char *var, char *val) {
 	char *input = NULL;
@@ -6832,6 +6859,9 @@
 		ocount[i] = (unsigned int)skiplist_num_items(xobject_skiplists[i]);
 	}
 
+	/* commandenvironments are based on commands */
+	ocount[COMMANDENVIRONMENT_SKIPLIST] = skiplist_num_items(xobject_skiplists[COMMAND_SKIPLIST]);
+
 	/* dependencies are handled specially */
 	ocount[SERVICEDEPENDENCY_SKIPLIST] = 0;
 	ocount[HOSTDEPENDENCY_SKIPLIST] = 0;
@@ -7136,6 +7166,9 @@
 /* registers a command definition */
 int xodtemplate_register_command(xodtemplate_command *this_command) {
 	command *new_command = NULL;
+#ifndef NSCGI
+	xodtemplate_commandenvironment *command_environment;
+#endif
 
 	/* bail out if we shouldn't register this object */
 	if(this_command->register_object == FALSE)
@@ -7150,6 +7183,16 @@
 		return ERROR;
 		}
 
+#ifndef NSCGI
+	/* add all the environment lines */
+	for(command_environment = this_command->env; command_environment!=NULL; command_environment=command_environment->next) {
+		if(add_command_environment_to_command(new_command, command_environment->env)==NULL) {
+			logit(NSLOG_CONFIG_ERROR, TRUE, "Error: Could not add environment '%s' to command (config file '%s', starting on line %d)\n", command_environment->env, xodtemplate_config_file_name(this_command->_config_file), this_command->_start_line);
+			return ERROR;
+			}
+		}
+#endif
+
 	return OK;
 	}
 
@@ -8314,6 +8357,8 @@
 	xodtemplate_daterange *next_daterange = NULL;
 	xodtemplate_command *this_command = NULL;
 	xodtemplate_command *next_command = NULL;
+	xodtemplate_commandenvironment *this_command_env = NULL;
+	xodtemplate_commandenvironment *next_command_env = NULL;
 	xodtemplate_contactgroup *this_contactgroup = NULL;
 	xodtemplate_contactgroup *next_contactgroup = NULL;
 	xodtemplate_hostgroup *this_hostgroup = NULL;
@@ -8360,6 +8405,15 @@
 		my_free(this_command->name);
 		my_free(this_command->command_name);
 		my_free(this_command->command_line);
+
+		this_command_env = this_command->env;
+		while(this_command_env != NULL) {
+			next_command_env = this_command_env->next;
+			my_free(this_command_env->env);
+			my_free(this_command_env);
+			this_command_env = next_command_env;
+			}
+
 		my_free(this_command);
 		}
 	xodtemplate_command_list = NULL;
Index: xdata/xodtemplate.h
===================================================================
--- xdata/xodtemplate.h	(revision 2574)
+++ xdata/xodtemplate.h	(working copy)
@@ -68,6 +68,11 @@
 	struct xodtemplate_customvariablesmember_struct *next;
 	} xodtemplate_customvariablesmember;
 
+/* COMMANDENVIRONMENT structure */
+typedef struct xodtemplate_commandenvironment_struct {
+	char    *env;
+	struct xodtemplate_commandenvironment_struct *next;
+	} xodtemplate_commandenvironment;
 
 /* DATERANGE structure */
 typedef struct xodtemplate_daterange_struct {
@@ -116,6 +121,7 @@
 
     char       *command_name;
     char       *command_line;
+    xodtemplate_commandenvironment *env;
 
     unsigned has_been_resolved : 1;
     unsigned register_object : 1;
@@ -787,6 +793,7 @@
 xodtemplate_customvariablesmember *xodtemplate_add_custom_variable_to_service(xodtemplate_service *, char *, char *);
 xodtemplate_customvariablesmember *xodtemplate_add_custom_variable_to_contact(xodtemplate_contact *, char *, char *);
 xodtemplate_customvariablesmember *xodtemplate_add_custom_variable_to_object(xodtemplate_customvariablesmember **, char *, char *);
+xodtemplate_commandenvironment *xodtemplate_add_commandenvironment(xodtemplate_command *, char *);
 
 
 int xodtemplate_register_objects(void);
Index: lib/kvvec.h
===================================================================
--- lib/kvvec.h	(revision 2574)
+++ lib/kvvec.h	(working copy)
@@ -144,6 +144,7 @@
  * @return 0 on success, < 0 on errors
  */
 extern int kvvec_destroy(struct kvvec *kvv, int flags);
+void kvvec_free_kvpairs(struct kvvec *kvv, int flags);
 
 /**
  * Create a linear buffer of all the key/value pairs and
Index: lib/worker.c
===================================================================
--- lib/worker.c	(revision 2574)
+++ lib/worker.c	(working copy)
@@ -254,6 +254,8 @@
 		struct key_value *kv = &cp->request->kv[i];
 		/* skip environment macros */
 		if (kv->key_len == 3 && !strcmp(kv->key, "env")) {
+			char *var = strtok(kv->value, "=");
+			unsetenv(var);
 			continue;
 		}
 		kvvec_addkv_wlen(&resp, kv->key, kv->key_len, kv->value, kv->value_len);
@@ -500,6 +502,11 @@
 			cp->timeout = (unsigned int)strtoul(value, &endptr, 0);
 			continue;
 		}
+		if (!strcmp(key, "env")) {
+			// TODO: Should confirm if putenv is available
+			putenv(value);
+			continue;
+		}
 	}
 
 	/* jobs without a timeout get a default of 60 seconds. */
Index: include/objects.h
===================================================================
--- include/objects.h	(revision 2574)
+++ include/objects.h	(working copy)
@@ -45,7 +45,7 @@
 
 /***************** SKIP LISTS ****************/
 
-#define NUM_OBJECT_SKIPLISTS                   12
+#define NUM_OBJECT_SKIPLISTS                   13
 #define NUM_HASHED_OBJECT_TYPES                8
 
 #define HOST_SKIPLIST                          0
@@ -60,6 +60,7 @@
 #define SERVICEDEPENDENCY_SKIPLIST             9
 #define HOSTESCALATION_SKIPLIST                10
 #define SERVICEESCALATION_SKIPLIST             11
+#define COMMANDENVIRONMENT_SKIPLIST            12
 
 
 /***************** DATE RANGE TYPES *******************/
@@ -314,6 +315,12 @@
 	} customvariablesmember;
 
 
+/* COMMANDENVIRONMENT structure */
+typedef struct commandenvironment {
+	char    *env;
+	struct commandenvironment *next;
+	} commandenvironment;
+
 /* COMMAND structure */
 typedef struct command {
 	unsigned int id;
@@ -736,6 +743,7 @@
 extern struct hostescalation *hostescalation_list;
 extern struct serviceescalation *serviceescalation_list;
 extern struct command **command_ary;
+extern struct commandenvironment **commandenvironment_ary;
 extern struct timeperiod **timeperiod_ary;
 extern struct host **host_ary;
 extern struct service **service_ary;
@@ -759,6 +767,7 @@
 struct contact *add_contact(char *name, char *alias, char *email, char *pager, char **addresses, char *svc_notification_period, char *host_notification_period, int service_notification_options, int host_notification_options, int service_notifications_enabled, int host_notifications_enabled, int can_submit_commands, int retain_status_information, int retain_nonstatus_information, unsigned int minimum_value);
 struct commandsmember *add_service_notification_command_to_contact(contact *, char *);				/* adds a service notification command to a contact definition */
 struct commandsmember *add_host_notification_command_to_contact(contact *, char *);				/* adds a host notification command to a contact definition */
+struct commandenvironment *add_command_environment_to_command(command *, char *);				/* adds a environment string to a command */
 struct customvariablesmember *add_custom_variable_to_contact(contact *, char *, char *);                       /* adds a custom variable to a service definition */
 struct host *add_host(char *name, char *display_name, char *alias, char *address, char *check_period, int initial_state, double check_interval, double retry_interval, int max_attempts, int notification_options, double notification_interval, double first_notification_delay, char *notification_period, int notifications_enabled, char *check_command, int checks_enabled, int accept_passive_checks, char *event_handler, int event_handler_enabled, int flap_detection_enabled, double low_flap_threshold, double high_flap_threshold, int flap_detection_options, int stalking_options, int process_perfdata, int check_freshness, int freshness_threshold, char *notes, char *notes_url, char *action_url, char *icon_image, char *icon_image_alt, char *vrml_image, char *statusmap_image, int x_2d, int y_2d, int have_2d_coords, double x_3d, double y_3d, double z_3d, int have_3d_coords, int should_be_drawn, int retain_status_information, int retain_nonstatus_information, int obsess_over_host, unsigned int hourly_value);
 struct hostsmember *add_parent_host_to_host(host *, char *);							/* adds a parent host to a host definition */
Index: include/workers.h
===================================================================
--- include/workers.h	(revision 2574)
+++ include/workers.h	(working copy)
@@ -25,10 +25,10 @@
 extern void free_worker_memory(int flags);
 extern int workers_alive(void);
 extern int init_workers(int desired_workers);
-extern int wproc_run_check(check_result *cr, char *cmd, nagios_macros *mac);
-extern int wproc_notify(char *cname, char *hname, char *sdesc, char *cmd, nagios_macros *mac);
-extern int wproc_run(int job_type, char *cmd, int timeout, nagios_macros *mac);
-extern int wproc_run_service_job(int jtype, int timeout, service *svc, char *cmd, nagios_macros *mac);
-extern int wproc_run_host_job(int jtype, int timeout, host *hst, char *cmd, nagios_macros *mac);
+extern int wproc_run_check(check_result *cr, char *cmd, struct kvvec *kvv);
+extern int wproc_notify(char *cname, char *hname, char *sdesc, char *cmd, struct kvvec *kvv);
+extern int wproc_run(int job_type, char *cmd, int timeout, struct kvvec *kvv);
+extern int wproc_run_service_job(int jtype, int timeout, service *svc, char *cmd, struct kvvec *kvv);
+extern int wproc_run_host_job(int jtype, int timeout, host *hst, char *cmd, struct kvvec *kvv);
 extern int wproc_destroy(worker_process *wp, int flags);
 #endif
Index: include/nagios.h
===================================================================
--- include/nagios.h	(revision 2574)
+++ include/nagios.h	(working copy)
@@ -431,6 +431,7 @@
 
 /******************** FUNCTIONS **********************/
 extern int set_loadctl_options(char *opts, unsigned int len);
+extern int add_command_environment_to_kvv(struct kvvec *kvv, struct command *check_command, nagios_macros *mac);
 
 /* silly helpers useful pretty much all over the place */
 extern const char *service_state_name(int state);
Index: include/macros.h
===================================================================
--- include/macros.h	(revision 2574)
+++ include/macros.h	(working copy)
@@ -224,6 +224,7 @@
 #define STRIP_ILLEGAL_MACRO_CHARS       1
 #define ESCAPE_MACRO_CHARS              2
 #define URL_ENCODE_MACRO_CHARS		4
+#define SET_EMPTY_IF_INVALID        8
 
 
 
Index: sample-config/template-object/commands.cfg.in
===================================================================
--- sample-config/template-object/commands.cfg.in	(revision 2574)
+++ sample-config/template-object/commands.cfg.in	(working copy)
@@ -36,6 +36,16 @@
 	}
 
 
+# You can also define environment variables that will be passed to the command after expansion
+# Unknown/invalid macros will get converted to an empty string (eg, service macros for a host check)
+# This applies for checks, notifications, event handlers and performance handlers
+define command{
+	command_name	custom-notify-by-carrier-pigeon
+	command_line	$USER1$/notify-script
+	env NAGIOS_SERVICESTATE=$SERVICESTATE$
+	env NAGIOS_HOSTALIAS=$HOSTALIAS$
+	env ORACLE_SID=db1
+	}
 
 
 
Index: common/macros.c
===================================================================
--- common/macros.c	(revision 2574)
+++ common/macros.c	(working copy)
@@ -189,7 +189,8 @@
 				}
 
 			/* a non-macro, just some user-defined string between two $s */
-			else {
+			/* The SET_EMPTY_IF_INVALID option allows this to be empty values, simulating the old envvars behaviour */
+			else if (!(options & SET_EMPTY_IF_INVALID)) {
 				log_debug_info(DEBUGL_MACROS, 2, "  Non-macro.  Running output (%lu): '%s'\n", (unsigned long)strlen(*output_buffer), *output_buffer);
 
 				/* add the plain text to the end of the already processed buffer */
Index: common/objects.c
===================================================================
--- common/objects.c	(revision 2574)
+++ common/objects.c	(working copy)
@@ -68,6 +68,7 @@
 serviceescalation **serviceescalation_ary = NULL;
 hostdependency **hostdependency_ary = NULL;
 servicedependency **servicedependency_ary = NULL;
+commandenvironment **commandenvironment_ary = NULL;
 
 #ifdef NSCORE
 int __nagios_object_structure_version = CURRENT_OBJECT_STRUCTURE_VERSION;
@@ -372,6 +373,8 @@
 		return ERROR;
 	if (mktable(command, COMMAND_SKIPLIST) != OK)
 		return ERROR;
+	if (mktable(commandenvironment, COMMANDENVIRONMENT_SKIPLIST) != OK)
+		return ERROR;
 	if (mktable(host, HOST_SKIPLIST) != OK)
 		return ERROR;
 	if (mktable(service, SERVICE_SKIPLIST) != OK)
@@ -1721,11 +1724,44 @@
 	command_ary[new_command->id] = new_command;
 	if(new_command->id)
 		command_ary[new_command->id - 1]->next = new_command;
+
+#ifndef NSCGI
+	// Reset commandenvironment
+	commandenvironment_ary[new_command->id] = NULL;
+#endif
 	return new_command;
 	}
 
 
+commandenvironment *add_command_environment_to_command(command *this_command, char *envstring) {
+	commandenvironment *new_command_environment = NULL;
 
+	if((envstring == NULL || !strcmp(envstring, ""))){
+		logit(NSLOG_CONFIG_ERROR, TRUE, "Error: Env string of command is NULL\n");
+		return NULL;
+		}
+
+	new_command_environment = calloc(1, sizeof(*new_command_environment));
+	if(!new_command_environment)
+		return NULL;
+
+	if((new_command_environment->env = (char *)strdup(envstring)) == NULL) {
+        free(new_command_environment);
+		return NULL;
+        }
+
+	// Add this environment to head of list, indexed by command->id
+	// There will be a linked list of environment values
+	if(commandenvironment_ary[this_command->id]!=NULL){
+		new_command_environment->next = commandenvironment_ary[this_command->id];
+	} else {
+		new_command_environment->next = NULL;
+	}
+	commandenvironment_ary[this_command->id] = new_command_environment;
+
+	return new_command_environment;
+}
+
 /* add a new service escalation to the list in memory */
 serviceescalation *add_serviceescalation(char *host_name, char *description, int first_notification, int last_notification, double notification_interval, char *escalation_period, int escalation_options) {
 	serviceescalation *new_serviceescalation = NULL;
@@ -2557,6 +2593,10 @@
 	customvariablesmember *next_customvariablesmember = NULL;
 	commandsmember *this_commandsmember = NULL;
 	commandsmember *next_commandsmember = NULL;
+#ifndef NSCGI
+	commandenvironment *this_command_environment = NULL;
+	commandenvironment *next_command_environment = NULL;
+#endif
 	register int x = 0;
 	register int i = 0;
 
@@ -2887,6 +2927,15 @@
 	/**** free command memory ****/
 	for (i = 0; i < num_objects.commands; i++) {
 		command *this_command = command_ary[i];
+
+#ifndef NSCGI
+		next_command_environment = commandenvironment_ary[this_command->id];
+		while((this_command_environment = next_command_environment)!=NULL) {
+			next_command_environment=this_command_environment->next;
+			my_free(this_command_environment->env);
+			my_free(this_command_environment);
+		}
+#endif
 		my_free(this_command->name);
 		my_free(this_command->command_line);
 		my_free(this_command);
@@ -3119,8 +3168,16 @@
 
 void fcache_command(FILE *fp, command *temp_command)
 {
-	fprintf(fp, "define command {\n\tcommand_name\t%s\n\tcommand_line\t%s\n\t}\n\n",
-		   temp_command->name, temp_command->command_line);
+	commandenvironment *temp_command_environment;
+	fprintf(fp, "define command {\n\tcommand_name\t%s\n\tcommand_line\t%s\n",
+		temp_command->name, temp_command->command_line);
+
+	temp_command_environment = commandenvironment_ary[temp_command->id];
+	while(temp_command_environment!=NULL) {
+		fprintf(fp, "\tenv\t%s\n", temp_command_environment->env);
+		temp_command_environment=temp_command_environment->next;
+		}
+	fprintf(fp, "\t}\n\n");
 }
 
 void fcache_contactgroup(FILE *fp, contactgroup *temp_contactgroup)
Index: base/utils.c
===================================================================
--- base/utils.c	(revision 2574)
+++ base/utils.c	(working copy)
@@ -244,6 +244,23 @@
 	}
 
 
+int add_command_environment_to_kvv(struct kvvec *kvv, struct command *check_command, nagios_macros *mac) {
+    char *temp_envvar = NULL;
+    commandenvironment *temp_command_environment;
+	
+    /* Process macros in the command env settings */
+    temp_command_environment=commandenvironment_ary[check_command->id];
+    while(temp_command_environment!=NULL) {
+        process_macros_r(mac, temp_command_environment->env, &temp_envvar, SET_EMPTY_IF_INVALID);
+        kvvec_addkv(kvv, "env", strdup(temp_envvar));
+        free(temp_envvar);
+        temp_command_environment=temp_command_environment->next;
+		}
+
+	return TRUE;
+
+}
+
 int set_loadctl_options(char *opts, unsigned int len)
 {
 	struct kvvec *kvv;
Index: base/checks.c
===================================================================
--- base/checks.c	(revision 2574)
+++ base/checks.c	(working copy)
@@ -155,6 +155,7 @@
 	double old_latency = 0.0;
 	check_result *cr;
 	int runchk_result = OK;
+	static struct kvvec kvv = KVVEC_INITIALIZER;
 #ifdef USE_EVENT_BROKER
 	int neb_result = OK;
 #endif
@@ -173,6 +174,10 @@
 	if((temp_host = svc->host_ptr) == NULL)
 		return ERROR;
 
+	/* Setup key vectors, for passing extra data to worker */
+	if (!kvvec_init(&kvv,0))
+		return ERROR;
+
 	/******** GOOD TO GO FOR A REAL SERVICE CHECK AT THIS POINT ********/
 
 #ifdef USE_EVENT_BROKER
@@ -239,6 +244,9 @@
 		return ERROR;
 		}
 
+	/* Process macros in the command env settings */
+	add_command_environment_to_kvv(&kvv, svc->check_command_ptr, &mac);
+
 	/* get the command start time */
 	gettimeofday(&start_time, NULL);
 
@@ -294,12 +302,13 @@
 	update_check_stats((scheduled_check == TRUE) ? ACTIVE_SCHEDULED_SERVICE_CHECK_STATS : ACTIVE_ONDEMAND_SERVICE_CHECK_STATS, start_time.tv_sec);
 
 	/* paw off the check to a worker to run */
-	runchk_result = wproc_run_check(cr, processed_command, &mac);
+	runchk_result = wproc_run_check(cr, processed_command, &kvv);
 	if (runchk_result == ERROR) {
 		logit(NSLOG_RUNTIME_ERROR, TRUE, "Unable to run check for service '%s' on host '%s'\n", svc->description, svc->host_name);
 	}
 
 	/* free memory */
+	kvvec_free_kvpairs(&kvv, KVVEC_FREE_VALUES);
 	my_free(processed_command);
 	clear_volatile_macros_r(&mac);
 
@@ -2291,6 +2300,7 @@
 	double old_latency = 0.0;
 	check_result *cr;
 	int runchk_result = OK;
+	static struct kvvec kvv = KVVEC_INITIALIZER;
 #ifdef USE_EVENT_BROKER
 	int neb_result = OK;
 #endif
@@ -2313,6 +2323,10 @@
 		return ERROR;
 		}
 
+	/* Setup key vectors, for passing extra data to worker */
+	if (!kvvec_init(&kvv,0))
+		return ERROR;
+
 	/******** GOOD TO GO FOR A REAL HOST CHECK AT THIS POINT ********/
 
 #ifdef USE_EVENT_BROKER
@@ -2370,6 +2384,9 @@
 		return ERROR;
 		}
 
+	/* Process macros in the command env settings */
+	add_command_environment_to_kvv(&kvv, hst->check_command_ptr, &mac);
+
 	/* get the command start time */
 	gettimeofday(&start_time, NULL);
 
@@ -2415,12 +2432,13 @@
 	update_check_stats((scheduled_check == TRUE) ? ACTIVE_SCHEDULED_HOST_CHECK_STATS : ACTIVE_ONDEMAND_HOST_CHECK_STATS, start_time.tv_sec);
 	update_check_stats(PARALLEL_HOST_CHECK_STATS, start_time.tv_sec);
 
-	runchk_result = wproc_run_check(cr, processed_command, &mac);
+	runchk_result = wproc_run_check(cr, processed_command, &kvv);
 	if (runchk_result == ERROR) {
 		logit(NSLOG_RUNTIME_ERROR, TRUE, "Unable to run check for host '%s'\n", hst->name);
 	}
 
 	/* free memory */
+	kvvec_free_kvpairs(&kvv,KVVEC_FREE_VALUES);
 	clear_volatile_macros_r(&mac);
 	my_free(processed_command);
 
Index: base/notifications.c
===================================================================
--- base/notifications.c	(revision 2574)
+++ base/notifications.c	(working copy)
@@ -690,6 +690,7 @@
 	struct timeval method_start_time, method_end_time;
 	int macro_options = STRIP_ILLEGAL_MACRO_CHARS | ESCAPE_MACRO_CHARS;
 	int neb_result;
+	static struct kvvec kvv = KVVEC_INITIALIZER;
 
 	log_debug_info(DEBUGL_FUNCTIONS, 0, "notify_contact_of_service()\n");
 
@@ -726,6 +727,9 @@
 			continue ;
 #endif
 
+		if(!kvvec_init(&kvv,0))
+			continue;
+
 		/* get the raw command line */
 		get_raw_command_line_r(mac, temp_commandsmember->command_ptr, temp_commandsmember->command, &raw_command, macro_options);
 		if(raw_command == NULL)
@@ -786,10 +790,13 @@
 			my_free(processed_buffer);
 			}
 
+		add_command_environment_to_kvv(&kvv, temp_commandsmember->command_ptr, mac);
+
 		/* run the notification command */
-		wproc_notify(cntct->name, svc->host_name, svc->description, processed_command, mac);
+		wproc_notify(cntct->name, svc->host_name, svc->description, processed_command, &kvv);
 
 		/* free memory */
+		kvvec_free_kvpairs(&kvv, KVVEC_FREE_VALUES);
 		my_free(command_name);
 		my_free(processed_command);
 
@@ -1588,6 +1595,7 @@
 	struct timeval method_end_time;
 	int macro_options = STRIP_ILLEGAL_MACRO_CHARS | ESCAPE_MACRO_CHARS;
 	int neb_result;
+	static struct kvvec kvv = KVVEC_INITIALIZER;
 
 
 	log_debug_info(DEBUGL_FUNCTIONS, 0, "notify_contact_of_host()\n");
@@ -1625,6 +1633,9 @@
 			continue ;
 #endif
 
+		if(!kvvec_init(&kvv,0))
+			continue;
+
 		/* get the raw command line */
 		get_raw_command_line_r(mac, temp_commandsmember->command_ptr, temp_commandsmember->command, &raw_command, macro_options);
 		if(raw_command == NULL)
@@ -1685,12 +1696,15 @@
 			my_free(processed_buffer);
 			}
 
+		add_command_environment_to_kvv(&kvv, temp_commandsmember->command_ptr, mac);
+
 		/* run the notification command */
-		wproc_notify(cntct->name, hst->name, NULL, processed_command, mac);
+		wproc_notify(cntct->name, hst->name, NULL, processed_command, &kvv);
 
 		/* @todo Handle nebmod stuff when getting results from workers */
 
 		/* free memory */
+		kvvec_free_kvpairs(&kvv, KVVEC_FREE_VALUES);
 		my_free(command_name);
 		my_free(processed_command);
 
Index: base/sehandlers.c
===================================================================
--- base/sehandlers.c	(revision 2574)
+++ base/sehandlers.c	(working copy)
@@ -47,6 +47,7 @@
 	host *temp_host = NULL;
 	int macro_options = STRIP_ILLEGAL_MACRO_CHARS | ESCAPE_MACRO_CHARS;
 	nagios_macros mac;
+	static struct kvvec kvv = KVVEC_INITIALIZER;
 
 	log_debug_info(DEBUGL_FUNCTIONS, 0, "obsessive_compulsive_service_check_processor()\n");
 
@@ -65,6 +66,9 @@
 	if((temp_host = (host *)svc->host_ptr) == NULL)
 		return ERROR;
 
+	if(!kvvec_init(&kvv,0))
+		return ERROR;
+
 	/* update service macros */
 	memset(&mac, 0, sizeof(mac));
 	grab_host_macros_r(&mac, temp_host);
@@ -89,10 +93,13 @@
 
 	log_debug_info(DEBUGL_CHECKS, 2, "Processed obsessive compulsive service processor command line: %s\n", processed_command);
 
+	add_command_environment_to_kvv(&kvv, ocsp_command_ptr, &mac);
+
 	/* run the command through a worker */
-	wproc_run_service_job(WPJOB_OCSP, ocsp_timeout, svc, processed_command, &mac);
+	wproc_run_service_job(WPJOB_OCSP, ocsp_timeout, svc, processed_command, &kvv);
 
 	/* free memory */
+	kvvec_free_kvpairs(&kvv, KVVEC_FREE_VALUES);
 	clear_volatile_macros_r(&mac);
 	my_free(processed_command);
 
@@ -107,6 +114,7 @@
 	char *processed_command = NULL;
 	int macro_options = STRIP_ILLEGAL_MACRO_CHARS | ESCAPE_MACRO_CHARS;
 	nagios_macros mac;
+	static struct kvvec kvv = KVVEC_INITIALIZER;
 
 	log_debug_info(DEBUGL_FUNCTIONS, 0, "obsessive_compulsive_host_check_processor()\n");
 
@@ -121,6 +129,9 @@
 	if(ochp_command == NULL)
 		return ERROR;
 
+	if(!kvvec_init(&kvv,0))
+		return ERROR;
+
 	/* update macros */
 	memset(&mac, 0, sizeof(mac));
 	grab_host_macros_r(&mac, hst);
@@ -144,10 +155,13 @@
 
 	log_debug_info(DEBUGL_CHECKS, 2, "Processed obsessive compulsive host processor command line: %s\n", processed_command);
 
+	add_command_environment_to_kvv(&kvv, ochp_command_ptr, &mac);
+
 	/* run the command through a worker */
-	wproc_run_host_job(WPJOB_OCHP, ochp_timeout, hst, processed_command, &mac);
+	wproc_run_host_job(WPJOB_OCHP, ochp_timeout, hst, processed_command, &kvv);
 
 	/* free memory */
+	kvvec_free_kvpairs(&kvv, KVVEC_FREE_VALUES);
 	clear_volatile_macros_r(&mac);
 	my_free(processed_command);
 
@@ -221,6 +235,7 @@
 	int neb_result = OK;
 #endif
 	int macro_options = STRIP_ILLEGAL_MACRO_CHARS | ESCAPE_MACRO_CHARS;
+	static struct kvvec kvv = KVVEC_INITIALIZER;
 
 
 	log_debug_info(DEBUGL_FUNCTIONS, 0, "run_global_service_event_handler()\n");
@@ -236,6 +251,9 @@
 	if(global_service_event_handler == NULL)
 		return ERROR;
 
+	if(!kvvec_init(&kvv,0))
+		return ERROR;
+
 	log_debug_info(DEBUGL_EVENTHANDLERS, 1, "Running global event handler for service '%s' on host '%s'...\n", svc->description, svc->host_name);
 
 #ifdef USE_EVENT_BROKER
@@ -280,9 +298,11 @@
 		}
 #endif
 
+	add_command_environment_to_kvv(&kvv, global_service_event_handler_ptr, mac);
+
 	/* run the command through a worker */
 	/* XXX FIXME make base/workers.c handle the eventbroker stuff below */
-	result = wproc_run(WPJOB_GLOBAL_SVC_EVTHANDLER, processed_command, event_handler_timeout, mac);
+	result = wproc_run(WPJOB_GLOBAL_SVC_EVTHANDLER, processed_command, event_handler_timeout, &kvv);
 
 	/* check to see if the event handler timed out */
 	if(early_timeout == TRUE)
@@ -299,6 +319,7 @@
 #endif
 
 	/* free memory */
+	kvvec_free_kvpairs(&kvv, KVVEC_FREE_VALUES);
 	my_free(command_output);
 	my_free(processed_command);
 	my_free(raw_logentry);
@@ -325,6 +346,7 @@
 	int neb_result = OK;
 #endif
 	int macro_options = STRIP_ILLEGAL_MACRO_CHARS | ESCAPE_MACRO_CHARS;
+	static struct kvvec kvv = KVVEC_INITIALIZER;
 
 
 	log_debug_info(DEBUGL_FUNCTIONS, 0, "run_service_event_handler()\n");
@@ -336,6 +358,9 @@
 	if(svc->event_handler == NULL)
 		return ERROR;
 
+	if(!kvvec_init(&kvv,0))
+		return ERROR;
+
 	log_debug_info(DEBUGL_EVENTHANDLERS, 1, "Running event handler for service '%s' on host '%s'...\n", svc->description, svc->host_name);
 
 #ifdef USE_EVENT_BROKER
@@ -380,9 +405,11 @@
 		}
 #endif
 
+	add_command_environment_to_kvv(&kvv, svc->event_handler_ptr, mac);
+
 	/* run the command through a worker */
 	/* XXX FIXME make base/workers.c handle the eventbroker stuff below */
-	result = wproc_run(WPJOB_SVC_EVTHANDLER, processed_command, event_handler_timeout, mac);
+	result = wproc_run(WPJOB_SVC_EVTHANDLER, processed_command, event_handler_timeout, &kvv);
 
 	/* check to see if the event handler timed out */
 	if(early_timeout == TRUE)
@@ -399,6 +426,7 @@
 #endif
 
 	/* free memory */
+	kvvec_free_kvpairs(&kvv, KVVEC_FREE_VALUES);
 	my_free(command_output);
 	my_free(processed_command);
 	my_free(raw_logentry);
@@ -466,6 +494,7 @@
 	int neb_result = OK;
 #endif
 	int macro_options = STRIP_ILLEGAL_MACRO_CHARS | ESCAPE_MACRO_CHARS;
+	static struct kvvec kvv = KVVEC_INITIALIZER;
 
 
 	log_debug_info(DEBUGL_FUNCTIONS, 0, "run_global_host_event_handler()\n");
@@ -481,6 +510,9 @@
 	if(global_host_event_handler == NULL)
 		return ERROR;
 
+	if(!kvvec_init(&kvv,0))
+		return ERROR;
+
 	log_debug_info(DEBUGL_EVENTHANDLERS, 1, "Running global event handler for host '%s'..\n", hst->name);
 
 #ifdef USE_EVENT_BROKER
@@ -524,9 +556,11 @@
 		}
 #endif
 
+	add_command_environment_to_kvv(&kvv, global_host_event_handler_ptr, mac);
+
 	/* run the command through a worker */
 	/* XXX FIXME make base/workers.c handle the eventbroker stuff below */
-	wproc_run(WPJOB_GLOBAL_HOST_EVTHANDLER, processed_command, event_handler_timeout, mac);
+	wproc_run(WPJOB_GLOBAL_HOST_EVTHANDLER, processed_command, event_handler_timeout, &kvv);
 
 	/* check for a timeout in the execution of the event handler command */
 	if(early_timeout == TRUE)
@@ -543,6 +577,7 @@
 #endif
 
 	/* free memory */
+	kvvec_free_kvpairs(&kvv, KVVEC_FREE_VALUES);
 	my_free(command_output);
 	my_free(processed_command);
 	my_free(raw_logentry);
@@ -568,6 +603,7 @@
 	int neb_result = OK;
 #endif
 	int macro_options = STRIP_ILLEGAL_MACRO_CHARS | ESCAPE_MACRO_CHARS;
+	static struct kvvec kvv = KVVEC_INITIALIZER;
 
 
 	log_debug_info(DEBUGL_FUNCTIONS, 0, "run_host_event_handler()\n");
@@ -579,6 +615,9 @@
 	if(hst->event_handler == NULL)
 		return ERROR;
 
+	if(!kvvec_init(&kvv,0))
+		return ERROR;
+
 	log_debug_info(DEBUGL_EVENTHANDLERS, 1, "Running event handler for host '%s'..\n", hst->name);
 
 #ifdef USE_EVENT_BROKER
@@ -622,8 +661,10 @@
 		}
 #endif
 
+	add_command_environment_to_kvv(&kvv, hst->event_handler_ptr, mac);
+
 	/* run the command through a worker */
-	result = wproc_run(WPJOB_HOST_EVTHANDLER, processed_command, event_handler_timeout, mac);
+	result = wproc_run(WPJOB_HOST_EVTHANDLER, processed_command, event_handler_timeout, &kvv);
 
 	/* check to see if the event handler timed out */
 	if(early_timeout == TRUE)
@@ -640,6 +681,7 @@
 #endif
 
 	/* free memory */
+	kvvec_free_kvpairs(&kvv, KVVEC_FREE_VALUES);
 	my_free(command_output);
 	my_free(processed_command);
 	my_free(raw_logentry);
Index: base/workers.c
===================================================================
--- base/workers.c	(revision 2574)
+++ base/workers.c	(working copy)
@@ -881,12 +881,14 @@
  * as well as shipping the command off to a designated
  * worker
  */
-static int wproc_run_job(worker_job *job, nagios_macros *mac)
+static int wproc_run_job(worker_job *job, struct kvvec *inputkvv)
 {
 	static struct kvvec kvv = KVVEC_INITIALIZER;
 	struct kvvec_buf *kvvb;
+	struct key_value kv;
 	worker_process *wp;
 	int ret;
+	int i;
 
 	if (!job)
 		return ERROR;
@@ -901,13 +903,6 @@
 		return ERROR;
 	}
 
-	/*
-	 * XXX FIXME: add environment macros as
-	 *  kvvec_addkv(kvv, "env", "NAGIOS_LALAMACRO=VALUE");
-	 *  kvvec_addkv(kvv, "env", "NAGIOS_LALAMACRO2=VALUE");
-	 * so workers know to add them to environment. For now,
-	 * we don't support that though.
-	 */
 	if (!kvvec_init(&kvv, 4))	/* job_id, type, command and timeout */
 		return ERROR;
 
@@ -915,6 +910,17 @@
 	kvvec_addkv(&kvv, "type", (char *)mkstr("%d", job->type));
 	kvvec_addkv(&kvv, "command", job->command);
 	kvvec_addkv(&kvv, "timeout", (char *)mkstr("%u", job->timeout));
+	/*
+	 * Add environment macros, based on inputkvv
+	 */
+	for (i = 0; i < inputkvv->kv_pairs; i++) {
+		kv = inputkvv->kv[i];
+		if (!strcmp(kv.key, "env")) {
+			kvvec_addkv(&kvv, "env", kv.value);
+			continue;
+			}
+		}
+
 	kvvb = build_kvvec_buf(&kvv);
 	ret = write(wp->sd, kvvb->buf, kvvb->bufsize);
 	wp->jobs_running++;
@@ -947,7 +953,7 @@
 	return oj;
 }
 
-int wproc_notify(char *cname, char *hname, char *sdesc, char *cmd, nagios_macros *mac)
+int wproc_notify(char *cname, char *hname, char *sdesc, char *cmd, struct kvvec *kvv)
 {
 	worker_job *job;
 	wproc_object_job *oj;
@@ -957,10 +963,10 @@
 
 	job = create_job(WPJOB_NOTIFY, oj, notification_timeout, cmd);
 
-	return wproc_run_job(job, mac);
+	return wproc_run_job(job, kvv);
 }
 
-int wproc_run_service_job(int jtype, int timeout, service *svc, char *cmd, nagios_macros *mac)
+int wproc_run_service_job(int jtype, int timeout, service *svc, char *cmd, struct kvvec *kvv)
 {
 	worker_job *job;
 	wproc_object_job *oj;
@@ -970,10 +976,10 @@
 
 	job = create_job(jtype, oj, timeout, cmd);
 
-	return wproc_run_job(job, mac);
+	return wproc_run_job(job, kvv);
 }
 
-int wproc_run_host_job(int jtype, int timeout, host *hst, char *cmd, nagios_macros *mac)
+int wproc_run_host_job(int jtype, int timeout, host *hst, char *cmd, struct kvvec *kvv)
 {
 	worker_job *job;
 	wproc_object_job *oj;
@@ -983,10 +989,10 @@
 
 	job = create_job(jtype, oj, timeout, cmd);
 
-	return wproc_run_job(job, mac);
+	return wproc_run_job(job, kvv);
 }
 
-int wproc_run_check(check_result *cr, char *cmd, nagios_macros *mac)
+int wproc_run_check(check_result *cr, char *cmd, struct kvvec *kvv)
 {
 	worker_job *job;
 	time_t timeout;
@@ -997,14 +1003,14 @@
 		timeout = host_check_timeout;
 
 	job = create_job(WPJOB_CHECK, cr, timeout, cmd);
-	return wproc_run_job(job, mac);
+	return wproc_run_job(job, kvv);
 }
 
-int wproc_run(int jtype, char *cmd, int timeout, nagios_macros *mac)
+int wproc_run(int jtype, char *cmd, int timeout, struct kvvec *kvv)
 {
 	worker_job *job;
 	time_t real_timeout = timeout + time(NULL);
 
 	job = create_job(jtype, NULL, real_timeout, cmd);
-	return wproc_run_job(job, mac);
+	return wproc_run_job(job, kvv);
 }
